import java.io.*;
import java.util.*;
import java.net.URL;
import java.lang.*;
import java.math.*;

import tools.*;

/*------------------------------------------------------------------
 * CLASS:        compactSpectrum
 * LAST CHANGE:  15-12-2005
 * COMMENT:      conversion from the compact to matrix spectrum 
 *               representation and vice versa
 *
 * matrix representation: 
 * ----------------------
 * each row corresponds to a peak and is is represented as follows:
 * X_value "\t" Y_value
 *
 * sample: range = [30, 600]; increment = 1
 * 31	1020
 * 32	2847
 * 39	3632
 * 
 * 271	1661
 * 272	1614
 * 273	402
 *
 * compact representation: 
 * -----------------------
 * (<separator> <value>)+ <separator>
 *
 * <value>: a numerical value
 * <separator>: |, #, + or 
 *
 * assumptions:
 * - the 1st value of X-coordinate is known
 * - the increment for X-values is constant
 *
 * X-values are not explicitly stored. Instead, only the first value and 
 * the increment are stored, from which all other values can be generated. 
 * Y-values are stored in the order corresponding to the ascending order 
 * of X-values.
 *
 * | is used to separate Y-values corresponding to neighbouring X-values.
 *
 * When Y-values are missing for n successive X-values, # is used as a 
 * separator followed by the value of n.
 *
 * + and  are used as separators when the difference between the Y-value 
 * to be stored and the last stored Y-value is shorter (in the number of 
 * digits) than the actual Y-value to be stored. + and  are then used as 
 * separators followed by the absolute value of the difference and they 
 * indicate that the current Y-value can be obtained by adding or subtracting
 * the current value from the last recorded Y-value.
 *
 * sample:
 *
 * #1|1020|2847#6+785  |166147|402#327|
 *----------------------------------------------------------------*/

public class compactSpectrum
{

private static final int BASE = 10;

// *******************************************************************
//                        * * * MAIN * * *
// *******************************************************************
public static void main (String args[]) 
{
  try
  {
    String compact;

    // --- parameters ---
    EasyReader in = new EasyReader("input.txt");
    int x_min     = in.readInt();
    int x_max     = in.readInt();
    int increment = in.readInt();
    in.close();

    // --- PACKING ---
    System.out.print("Packing the spectrum given in the matrix.txt file... ");
    compact = pack("matrix.txt", x_min, x_max, increment);
    System.out.println("Done!");
    System.out.println("Compact spectrum representation:\n");
    System.out.println(compact + "\n");

    // --- UNPACKING ---
    EasyReader spectrum = new EasyReader("compact.txt");
    compact = spectrum.readString();
    spectrum.close();
    System.out.println("Unpacking the spectrum given in the compact.txt file...\n");
    unpack(compact, x_min, x_max, increment);
    System.out.println("\nDone!");
  }
  catch (java.lang.Exception ex) {ex.printStackTrace();}
}

// -------------------------------------------------------------------
// --- packs a matrix spectrum representation into a compact 
//     representation
//
// parameters:
//   matrixFile     - the file containing the matrix spectrum repr.
//   [x_min, x_max] - the range of x-values
//   increment      - the distance between x-points
// -------------------------------------------------------------------
private static String pack(String matrixFile, int x_min, int x_max, int increment)
{
  int index = x_min - increment; // --- indexes x-values
  int y_pre = 0;                 // --- the previous y-value
  String compact = "";           // --- compact spectrum
  String cell    = "";
  String skip    = "";

  try
  {
    EasyReader spectrum = new EasyReader(matrixFile); // --- input file

    while (!spectrum.eof())
    {
      String line = spectrum.readString();

      int from = 0;
      int to   = line.indexOf("\t", from);
      int x_value  = (new Integer(line.substring(from, to))).intValue();

      from = to + 1;
      int y_value  = (new Integer(line.substring(from))).intValue();

      if (x_value - index > increment) skip = "#" + (Integer.toString(x_value - index - increment, BASE).toUpperCase());
      else                             skip = "";

      int diff = y_value - y_pre;

      if (diff < 0) cell = "-" + (Integer.toString(-diff, BASE).toUpperCase());
      else          cell = "+" + (Integer.toString( diff, BASE).toUpperCase());

      String y_value_str = Integer.toString(y_value, BASE).toUpperCase();

      if (cell.length() > y_value_str.length()) cell = "|" + y_value_str;

      compact += skip + cell;
      y_pre = y_value;
      index = x_value;
    }

    if (index < x_max)
    {
      skip = "#" + (Integer.toString(x_max - index, BASE).toUpperCase());
      compact += skip;
    }

    compact += "|";
    spectrum.close();
  }
  catch (java.lang.Exception ex) {ex.printStackTrace();}

  return compact;
}
// -------------------------------------------------------------------



// -------------------------------------------------------------------
// --- unpacks a compact spectrum representation into a matrix 
//     representation (written to standard output)
//
// parameters:
//   compact        - compact spectrum representation
//   [x_min, x_max] - the range of x-values
//   increment      - the distance between x-points
// -------------------------------------------------------------------
private static void unpack(String compact, int x_min, int x_max, int increment)
{
  int index   = x_min - increment; // --- indexes x-values
  int y_pre   = 0;                 // --- the previous y-value
  int y_value = 0;

  int start = 1;
  int end   = 0;
  int end1  = 0;
  int length = compact.length();
  while (start < length)
  {
    end  = compact.indexOf('|', start);
    end1 = compact.indexOf('+', start);
    if (0 < end1 && end1 < end) end = end1;

    end1 = compact.indexOf('-', start);
    if (0 < end1 && end1 < end) end = end1;

    end1 = compact.indexOf('#', start);
    if (0 < end1 && end1 < end) end = end1;

    String extract = compact.substring(start, end);

    char separator = compact.charAt(start - 1);

    if (separator == '#')
    {
      int gap = Integer.parseInt(extract, BASE);

      index += increment * gap;
    }
    else
    {
      index += increment;

      y_value = Integer.parseInt(extract, BASE);

           if (separator == '+') y_value = y_pre + y_value;
      else if (separator == '-') y_value = y_pre - y_value;

      y_pre = y_value;

      System.out.println(index + "\t" + y_value);
    }

    start = end + 1;
  }
}
// -------------------------------------------------------------------
}
